import path from "path";
import { createRule } from "../createRule.js";
const rule = createRule({
  create(context) {
    return {
      "Program:exit"() {
        const original = JSON.parse(context.sourceCode.text);
        const { dependencies, devDependencies, peerDependencies } = original;
        const depObjs = [
          Object.entries(dependencies ?? {}),
          Object.entries(peerDependencies ?? {}),
          Object.entries(devDependencies ?? {})
        ];
        depObjs.forEach((obj) => {
          obj.forEach(([key, value]) => {
            const response = (localPath) => {
              const filePath = path.join(
                context.filename.replace(/package\.json/g, "/"),
                value.replace(new RegExp(localPath, "g"), ""),
                "/package.json"
              );
              try {
                if (!require.resolve(filePath)) {
                  context.report({
                    data: {
                      package: key,
                      path: value
                    },
                    messageId: "invalidPath",
                    node: context.sourceCode.ast
                  });
                }
              } catch {
                context.report({
                  data: {
                    package: key,
                    path: value
                  },
                  messageId: "invalidPath",
                  node: context.sourceCode.ast
                });
              }
            };
            if (value.startsWith("link:")) {
              response("link:");
            }
            if (value.startsWith("file:")) {
              response("file:");
            }
          });
        });
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Checks existence of local dependencies in the package.json",
      recommended: true
    },
    messages: {
      invalidPath: "The package {{package}} does not exist given the specified path: {{path}}."
    },
    schema: [],
    type: "problem"
  }
});
export {
  rule
};
//# sourceMappingURL=valid-local-dependency.mjs.map