import { PJV as PackageValidator } from "package-json-validator";
import { createRule } from "../createRule.js";
const unusedErrorPatterns = [
  /^Url not valid/i,
  /^Invalid version range for .+?: (?:file|npm|workspace):/i,
  /^author field should have name/i
];
const isUsableError = (errorText) => unusedErrorPatterns.every((pattern) => !pattern.test(errorText));
const rule = createRule({
  create(context) {
    return {
      "Program:exit"() {
        const validation = PackageValidator.validate(
          context.sourceCode.text
        );
        validation.errors?.filter(isUsableError).forEach((message) => {
          if (message) {
            context.report({
              // eslint-disable-next-line eslint-plugin/prefer-message-ids
              message,
              node: context.sourceCode.ast
            });
          }
        });
      }
    };
  },
  // eslint-disable-next-line eslint-plugin/prefer-message-ids
  meta: {
    docs: {
      category: "Best Practices",
      description: "Enforce that package.json has all properties required by the npm spec",
      recommended: true
    },
    schema: [],
    type: "problem"
  }
});
export {
  rule
};
//# sourceMappingURL=valid-package-definition.mjs.map